import cv2
import urllib.request
import numpy as np
import requests
import time

def nothing(x):
    pass

# Zmień adres IP poniżej zgodnie z IP pokazanym w monitorze szeregowym kodu Arduino
url = 'http://192.168.4.1/cam-mid.jpg'

# URL serwera ESP
esp_url = 'http://192.168.4.1'
sending_commands = False

cv2.namedWindow("live transmission", cv2.WINDOW_AUTOSIZE)
cv2.namedWindow("Tracking")
cv2.namedWindow("middle third")

cv2.createTrackbar("LH", "Tracking", 0, 255, nothing)
cv2.createTrackbar("LS", "Tracking", 0, 255, nothing)
cv2.createTrackbar("LV", "Tracking", 0, 255, nothing)
cv2.createTrackbar("UH", "Tracking", 255, 255, nothing)
cv2.createTrackbar("US", "Tracking", 255, 255, nothing)
cv2.createTrackbar("UV", "Tracking", 255, 255, nothing)

def turn_left(esp_url, amount):
    requests.get(f'{esp_url}/left?amount={amount}')
    print(f"Skręcam w lewo o wartość {amount}")

def turn_right(esp_url, amount):
    requests.get(f'{esp_url}/right?amount={amount}')
    print(f"Skręcam w prawo o wartość {amount}")

def go_straight(esp_url):
    requests.get(f'{esp_url}/straight')
    print("Jadę prosto")

def turn_left_hard(esp_url, amount):
    requests.get(f'{esp_url}/left_hard?amount={amount}')
    print(f"Duży skręt w lewo o wartość {amount}")

def turn_right_hard(esp_url, amount):
    requests.get(f'{esp_url}/right_hard?amount={amount}')
    print(f"Duży skręt w prawo o wartość {amount}")

while True:
    start_time = time.time()
    img_resp = urllib.request.urlopen(url)
    imgnp = np.array(bytearray(img_resp.read()), dtype=np.uint8)
    frame = cv2.imdecode(imgnp, -1)
    
    # Obracanie obrazu o 90 stopni w prawo
    frame = cv2.rotate(frame, cv2.ROTATE_90_CLOCKWISE)

    hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)

    l_h = cv2.getTrackbarPos("LH", "Tracking")
    l_s = cv2.getTrackbarPos("LS", "Tracking")
    l_v = cv2.getTrackbarPos("LV", "Tracking")

    u_h = cv2.getTrackbarPos("UH", "Tracking")
    u_s = cv2.getTrackbarPos("US", "Tracking")
    u_v = cv2.getTrackbarPos("UV", "Tracking")

    l_b = np.array([l_h, l_s, l_v])
    u_b = np.array([u_h, u_s, u_v])

    mask = cv2.inRange(hsv, l_b, u_b)
    
    # Poprawa maskowania za pomocą przetwarzania obrazu
    mask = cv2.GaussianBlur(mask, (5, 5), 0)
    mask = cv2.erode(mask, None, iterations=2)
    mask = cv2.dilate(mask, None, iterations=2)
    mask = cv2.medianBlur(mask, 5)
    
    res = cv2.bitwise_and(frame, frame, mask=mask)

    # Znajdowanie konturów dla całego kadru obrazu
    contours, _ = cv2.findContours(mask, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    if contours:
        c = max(contours, key=cv2.contourArea)
        x, y, w, h = cv2.boundingRect(c)
        cx_frame = x + w // 2
        cy_frame = y + h // 2
        
        # Rysowanie prostokąta wokół największego obszaru danego koloru
        cv2.rectangle(frame, (x, y), (x + w, y + h), (255, 0, 0), 2)
        
        if sending_commands:
            frame_height, frame_width, _ = frame.shape
            center_frame = frame_width // 2

            # Sterowanie proporcjonalne w zależności od pozycji środka konturu
            error = cx_frame - center_frame
            turn_amount = int(abs(error) // 10)
            
            if error > 40:  # Bardzo duży skręt w prawo
                turn_right_hard(esp_url, turn_amount)
            elif error > 10:  # Mały skręt w prawo
                turn_right(esp_url, turn_amount)
            elif error < -40:  # Bardzo duży skręt w lewo
                turn_left_hard(esp_url, turn_amount)
            elif error < -10:  # Mały skręt w lewo
                turn_left(esp_url, turn_amount)
            else:
                go_straight(esp_url)
                
    # Wyświetlanie podglądu
    cv2.imshow("live transmission", frame)
    cv2.imshow("mask", mask)
    cv2.imshow("res", res)

    key = cv2.waitKey(5)
    
    if key == ord('s'):
        sending_commands = True
        print("Rozpoczynanie wysyłania komend...")
    
    if key == ord('q'):
        break

    # Czekanie przez krótki czas przed następną iteracją
    elapsed_time = time.time() - start_time
    sleep_time = 0.3 - elapsed_time  # Cel na 0.2 sekundy na iterację
    if sleep_time > 0:
        time.sleep(sleep_time)

cv2.destroyAllWindows()
