#include <WebServer.h>
#include <WiFi.h>
#include <esp32cam.h>

const char* AP_SSID = "ESP32-CAM-AP";
const char* AP_PASSWORD = "esp32cam1234";

WebServer server(80);

static auto loRes = esp32cam::Resolution::find(320, 240);
static auto midRes = esp32cam::Resolution::find(350, 530);
static auto hiRes = esp32cam::Resolution::find(800, 600);

#define MOTOR_1_PIN_1    2 // silniki
#define MOTOR_1_PIN_2    14
#define MOTOR_2_PIN_1    12
#define MOTOR_2_PIN_2    13
#define LED_GPIO_NUM   4

void serveJpg() {
  auto frame = esp32cam::capture();
  if (frame == nullptr) {
    Serial.println("CAPTURE FAIL");
    server.send(503, "", "");
    return;
  }
  Serial.printf("CAPTURE OK %dx%d %db\n", frame->getWidth(), frame->getHeight(), static_cast<int>(frame->size()));

  server.setContentLength(frame->size());
  server.send(200, "image/jpeg");
  WiFiClient client = server.client();
  frame->writeTo(client);

  delay(100); 
}

void handleJpgLo() {
  if (!esp32cam::Camera.changeResolution(loRes)) {
    Serial.println("SET-LO-RES FAIL");
  }
  serveJpg();
}

void handleJpgHi() {
  if (!esp32cam::Camera.changeResolution(hiRes)) {
    Serial.println("SET-HI-RES FAIL");
  }
  serveJpg();
}

void handleJpgMid() {
  if (!esp32cam::Camera.changeResolution(midRes)) {
    Serial.println("SET-MID-RES FAIL");
  }
  serveJpg();
}

void setup() {
  Serial.begin(115200);
  Serial.println();

  // Konfiguracja punktu dostępowego WiFi
  WiFi.softAP(AP_SSID, AP_PASSWORD);
  IPAddress IP = WiFi.softAPIP();
  Serial.print("Punkt dostępowy WiFi został uruchomiony. Adres IP punktu dostępowego: ");
  Serial.println(IP);

  {
    using namespace esp32cam;
    Config cfg;
    cfg.setPins(pins::AiThinker);
    cfg.setResolution(hiRes);
    cfg.setBufferCount(2);
    cfg.setJpeg(80);

    bool ok = Camera.begin(cfg);
    Serial.println(ok ? "KAMERA OK" : "KAMERA FAIL");
  }

  // Dodanie obsługi URL-i do konsoli szeregowej
  Serial.println("Dostępne adresy URL:");
  Serial.println("http://" + IP.toString() + "/cam-lo.jpg");
  Serial.println("http://" + IP.toString() + "/cam-hi.jpg");
  Serial.println("http://" + IP.toString() + "/cam-mid.jpg");

  server.on("/cam-lo.jpg", handleJpgLo);
  server.on("/cam-hi.jpg", handleJpgHi);
  server.on("/cam-mid.jpg", handleJpgMid);

  server.begin();
  
  initRobot(); // inicjalizacja silników

  // Definiowanie tras
  server.on("/left", HTTP_GET, []() {
    if (server.hasArg("amount")) {
      int amount = server.arg("amount").toInt();
      CurveLeft(40, amount*4); // Obsługa skrętu w lewo
      Serial.println("Skręcam w lewo o wartość " + String(amount));
      server.send(200, "text/plain", "Skręcam w lewo");
    } else {
      server.send(400, "text/plain", "Brak parametru 'amount'");
    }
  });

  server.on("/right", HTTP_GET, []() {
    if (server.hasArg("amount")) {
      int amount = server.arg("amount").toInt();
      CurveRight(40, amount*4); // Obsługa skrętu w prawo
      Serial.println("Skręcam w prawo o wartość " + String(amount));
      server.send(200, "text/plain", "Skręcam w prawo");
    } else {
      server.send(400, "text/plain", "Brak parametru 'amount'");
    }
  });

  server.on("/straight", HTTP_GET, []() {
    goForward(100, 250); // Obsługa jazdy na wprost
    Serial.println("Jadę prosto");
    server.send(200, "text/plain", "Jadę prosto");
  });

  server.on("/left_hard", HTTP_GET, []() {
  if (server.hasArg("amount")) {
    int amount = server.arg("amount").toInt();
    CurveLeft(60, amount*4); // Obsługa dużego skrętu w lewo
    Serial.println("Duży skręt w lewo o wartość " + String(amount));
    server.send(200, "text/plain", "Duży skręt w lewo");
  } else {
    server.send(400, "text/plain", "Brak parametru 'amount'");
  }
});

server.on("/right_hard", HTTP_GET, []() {
  if (server.hasArg("amount")) {
    int amount = server.arg("amount").toInt();
    CurveRight(60, amount*4); // Obsługa dużego skrętu w prawo
    Serial.println("Duży skręt w prawo o wartość " + String(amount));
    server.send(200, "text/plain", "Duży skręt w prawo");
  } else {
    server.send(400, "text/plain", "Brak parametru 'amount'");
  }
});

}

void loop() {
  server.handleClient();
}

void initRobot() {
  pinMode(2, OUTPUT); // Motor pin //wolniejszy przód // lewe do przodu 
  pinMode(12, OUTPUT); // Motor pin // szybszy przód // prawe do porzodu
  pinMode(13, OUTPUT); // Motor pin // szybszy tył // prawe do tylu
  pinMode(14, OUTPUT); // Motor pin // wolniejszy przód // lewe do tylu
}

void goForward(int czas, int predkosc) {
  analogWrite(2, predkosc);  // Drive
  analogWrite(12, predkosc - 4);  // Drive
  delay(czas);
  stopEngine();
  
}

void goBack(int czas, int predkosc) {
  analogWrite(14, predkosc);  // Drive
  analogWrite(13, predkosc - 4);  // Drive
  delay(czas);
  stopEngine();
  
}

void goRight(int czas, int predkosc) {
  analogWrite(2, predkosc);  // Drive
  delay(czas);
  
}

void goLeft(int czas, int predkosc) {
  analogWrite(12, predkosc - 4);  // Drive
  delay(czas);
  stopEngine();
  
}

void turnRight(int czas, int predkosc) {
  analogWrite(2, predkosc);  // Drive
  analogWrite(13, predkosc - 4);  // Drive
  delay(czas);
  stopEngine();
}

void turnLeft(int czas, int predkosc) {
  analogWrite(12, predkosc - 4);  // Drive
  analogWrite(14, predkosc);  // Drive
  delay(czas);
  stopEngine();
  
}

void stopEngine() {
  analogWrite(2, 0);   // Stop
  analogWrite(12, 0);   // Stop
  analogWrite(13, 0);   // Stop
  analogWrite(14, 0);   // Stop
}

void setupLedFlash(int pin) {
  pinMode(pin, OUTPUT);
  digitalWrite(pin, HIGH); // Wstępnie ustaw stan na wysoki
}

void CurveRight(int czas, int turn){
  analogWrite(2, 255);  // Drive
  analogWrite(12, 255 - turn);  // Drive
  delay(czas);
  stopEngine();
}

void CurveLeft(int czas, int turn){
  analogWrite(12, 255);  // Drive
  analogWrite(2, 255 - turn);  // Drive
  delay(czas);
  stopEngine();
}
